package javax.management.openmbean;

import java.lang.reflect.Proxy;
import com.sun.jmx.mbeanserver.MXBeanMappingFactory;
import com.sun.jmx.mbeanserver.DefaultMXBeanMappingFactory;
import java.lang.reflect.Method;
import com.sun.jmx.mbeanserver.MXBeanLookup;
import java.lang.reflect.InvocationHandler;

public class CompositeDataInvocationHandler implements InvocationHandler
{
    private final CompositeData compositeData;
    private final MXBeanLookup lookup;
    
    public CompositeDataInvocationHandler(CompositeData compositeData) {
        this(compositeData, null);
    }
    
    CompositeDataInvocationHandler(CompositeData compositeData, MXBeanLookup lookup) {
        if (compositeData == null) {
            throw new IllegalArgumentException("compositeData");
        }
        this.compositeData = compositeData;
        this.lookup = lookup;
        return;
    }
    
    public CompositeData getCompositeData() {
        assert this.compositeData != null;
        return this.compositeData;
    }
    
    @Override
    public Object invoke(Object o, Method m, Object[] args) throws Throwable {
        String name = m.getName();
        if (m.getDeclaringClass() == Object.class) {
            if (name.equals("toString") && args == null) {
                return "Proxy[" + this.compositeData + "]";
            }
            if (name.equals("hashCode") && args == null) {
                return Integer.valueOf(this.compositeData.hashCode() + 1128548680);
            }
            if (name.equals("equals") && args.length == 1 && m.getParameterTypes()[0] == Object.class) {
                return Boolean.valueOf(this.equals(o, args[0]));
            }
            return m.invoke(this, args);
        }
        else {
            String propertyName = DefaultMXBeanMappingFactory.propertyName(m);
            if (propertyName == null) {
                throw new IllegalArgumentException("Method is not getter: " + m.getName());
            }
            Object o2;
            if (this.compositeData.containsKey(propertyName)) {
                o2 = this.compositeData.get(propertyName);
            }
            else {
                String decapitalize = DefaultMXBeanMappingFactory.decapitalize(propertyName);
                if (this.compositeData.containsKey(decapitalize)) {
                    o2 = this.compositeData.get(decapitalize);
                }
                else {
                    throw new IllegalArgumentException("No CompositeData item " + propertyName + (decapitalize.equals(propertyName) ? "" : (" or " + decapitalize)) + " to match " + name);
                }
            }
            return MXBeanMappingFactory.DEFAULT.mappingForType(m.getGenericReturnType(), MXBeanMappingFactory.DEFAULT).fromOpenValue(o2);
        }
    }
    
    private boolean equals(Object o, Object proxy) {
        if (proxy == null) {
            return false;
        }
        if (o.getClass() != proxy.getClass()) {
            return false;
        }
        InvocationHandler invocationHandler = Proxy.getInvocationHandler(proxy);
        return invocationHandler instanceof CompositeDataInvocationHandler && this.compositeData.equals(((CompositeDataInvocationHandler)invocationHandler).compositeData);
    }
}
